import java.util.ArrayList;

/**
 * DiagonalTLBR represents a collection of 9 squares on a diagonal (top left to botton right)
 * that wraps around from top to bottom, right to left as shown below.  Each diagonal
 * is numbered with an integer between 0 and 8 corresponding to the column number of the square
 * in row 0.
 */
public class DiagonalTLBR implements Section, Sudoku
{
    // _ _ _ x _ _ _ _ _   x _ _ _ _ _ _ _ _   _ _ _ _ _ _ _ _ x
    // _ _ _ _ x _ _ _ _   _ x _ _ _ _ _ _ _   x _ _ _ _ _ _ _ _
    // _ _ _ _ _ x _ _ _   _ _ x _ _ _ _ _ _   _ x _ _ _ _ _ _ _ 
    // _ _ _ _ _ _ x _ _   _ _ _ x _ _ _ _ _   _ _ x _ _ _ _ _ _ 
    // _ _ _ _ _ _ _ x _   _ _ _ _ x _ _ _ _   _ _ _ x _ _ _ _ _ 
    // _ _ _ _ _ _ _ _ x   _ _ _ _ _ x _ _ _   _ _ _ _ x _ _ _ _ 
    // x _ _ _ _ _ _ _ _   _ _ _ _ _ _ x _ _   _ _ _ _ _ x _ _ _ 
    // _ x _ _ _ _ _ _ _   _ _ _ _ _ _ _ x _   _ _ _ _ _ _ x _ _ 
    // _ _ x _ _ _ _ _ _   _ _ _ _ _ _ _ _ x   _ _ _ _ _ _ _ x _ 
    //  DiagonalTLBR 3      DiagonalTLBR 0      DiagonalTLBR 8
    //
    // Note: DiagonalTLBR 0 includes only the main diagonal.  For any square that is contained in row, column,
    // diagNum = (NUM_DIGITS + column - row) % NUM_DIGITS
    
    private int diagNum;
    Square[][] square;

    /**
     * @param diag int value 0 through 8, inclusive, that corresponds
     * to the column number of the square from the diagonal that is in row 0.
     */
    public DiagonalTLBR(int diag)
    {
        diagNum = diag;
    }

    public void update(Square[][] square, Square aSquare)
    {
      int squareRow = aSquare.getRow();
      int squareCol = aSquare.getCol();
      int diagColumn = (NUM_DIGITS + squareCol - squareRow) % NUM_DIGITS;
	  if ( diagNum == diagColumn )
	  {
	    for (int r = 0; r < NUM_DIGITS; r++)
	    {
	      int colNum = diagColumn % NUM_DIGITS;
		  if ( square[r][colNum].getValue() == 0 )
		  {
		    square[r][colNum].removePossibleValue( aSquare.getValue() );
		  }
		  diagColumn++;
        }
	  }
    }

   	public int[] getPair(Square[][] square)
   	{
      int diagColumn = diagNum;
	  for (int r = 0; r < NUM_DIGITS - 1; r++)
	  {
        int diagColumn2 = diagColumn + 1;
   	    Square sq = square[r][diagColumn % NUM_DIGITS];
	    for (int r2 = r + 1; r2 < NUM_DIGITS; r2++)
	    {
   	      Square sq2 = square[r2][diagColumn2 % NUM_DIGITS];
   	      if ( sq.equals( sq2 ) )
		  {
   	        ArrayList<Integer> list = sq.getPossibleValues();
            if ( list.size() == 2 )
            {
   	          int first = list.get(0);
   	          int second = list.get(1);
   	          int[] pair = new int[2];
   	          pair[0] = first;
   	          pair[1] = second;
   	          return pair;
   	        }
		  }
		  diagColumn2++;
	    }
		diagColumn++;
	  }
   	  return null;
   	}
	
	public void processPair(Square[][] square, int[] pair)
	{
      int diagColumn = diagNum;
	  for (int r = 0; r < NUM_DIGITS; r++)
	  {
  	    Square tempSquare = square[r][diagColumn % NUM_DIGITS];
	    ArrayList<Integer> list = tempSquare.getPossibleValues();
   	    if (list.size() > 1)
   	    {
	      int first = list.get(0);
   	      int second = list.get(1);
		  if ( list.size() != 2  ||
		       first  != pair[0] ||
		       second != pair[1] )
		  {
	        tempSquare.removePossibleValue( pair[0] );
	        tempSquare.removePossibleValue( pair[1] );
	      }
	    }
	    diagColumn++;
   	  }
	}

}
