import java.util.ArrayList;

/**
 * Region represents a 3 by 3 collection of squares on a Sudoku board
 * where the union of the regions is all squares on the Sudoku board and
 * the intersection of the regions is the empty set
 */
public class Region implements Section, Sudoku
{
    // regions are numbered left to right, top to bottom
    // 0   1   2
    // 3   4   5
    // 6   7   8
    private int regNum;
    Square[][] square;

    /**
     * @param reg int value 0 through 8, inclusive that corresponds
     * to a region, where regions are numbered as described above
     */
    public Region(int reg)
    {
        regNum = reg;
    }

    public void update(Square[][] square, Square aSquare){
      int squareRow = aSquare.getRow() / 3;
      int squareCol = aSquare.getCol() / 3;
      int regRow = regNum / 3;
      int regCol = regNum % 3;
      int rowStart = regRow * REGION_HEIGHT;
      int colStart = regCol * REGION_WIDTH;
      if ( squareRow == regRow && squareCol == regCol )
      {
	    for (int r = rowStart; r < rowStart + REGION_HEIGHT; r++){
		   for (int c = colStart; c < colStart + REGION_WIDTH; c++){
		      if ( square[r][c].getValue() == 0 )
		  	    square[r][c].removePossibleValue( aSquare.getValue() );
		   }
	    }
	  }
    }

   	public int[] getPair(Square[][] square)
   	{
      int regRow = regNum / 3;
      int regCol = regNum % 3;
      int rowStart = regRow * REGION_HEIGHT;
      int colStart = regCol * REGION_WIDTH;

      // regions are traversed left to right, top to bottom
      // regRow + 0,regCol + 0  regRow + 0,regCol + 1  regRow + 0,regCol + 2
      // regRow + 1,regCol + 0  regRow + 1,regCol + 1  regRow + 1,regCol + 2
      // regRow + 2,regCol + 0  regRow + 2,regCol + 1  regRow + 2,regCol + 2
   	  for (int pos = 0; pos < REGION_WIDTH * REGION_HEIGHT - 1; pos++)
   	  {
   	    // access squares from the region
   	    Square sq = square[rowStart + pos / 3][colStart + pos % 3];
     	for (int loc = pos+1; loc < REGION_WIDTH * REGION_HEIGHT; loc++)
     	{
   	      Square sq2 = square[rowStart + loc / 3][colStart + loc % 3];
   	      if ( sq.equals( sq2 ) )
   	      {
   	        ArrayList<Integer> list = sq.getPossibleValues();
            if ( list.size() == 2 )
            {
   	          int first = list.get(0);
   	          int second = list.get(1);
   	          int[] pair = new int[2];
   	          pair[0] = first;
   	          pair[1] = second;
   	          return pair;
   	        }
   	      }
   	    }
   	  }
   	  return null;
   	}
	
	public void processPair(Square[][] square, int[] pair)
	{
      int regRow = regNum / 3;
      int regCol = regNum % 3;
      int rowStart = regRow * REGION_HEIGHT;
      int colStart = regCol * REGION_WIDTH;
  	  for (int item = 0; item < REGION_WIDTH * REGION_HEIGHT; item++)
  	  {
  	    Square tempSquare = square[rowStart + item / 3][colStart + item % 3];
	    ArrayList<Integer> list = tempSquare.getPossibleValues();
   	    if (list.size() > 1)
   	    {
	      int first = list.get(0);
   	      int second = list.get(1);
		  if ( list.size() != 2  ||
		       first  != pair[0] ||
		       second != pair[1] )
		  {
	        tempSquare.removePossibleValue( pair[0] );
	        tempSquare.removePossibleValue( pair[1] );
	      }
	    }
   	  }
	}

}
