import java.util.ArrayList;

/**
 * Represents a square on the Sudoku board
 */
public class Square implements Sudoku
{
    // possible values for the square
	private ArrayList<Integer> possibleValues;
	// true if state has changes since last update; false otherwise
    private boolean stateChanged;
    // row in which the square resides
    private int row;
    // column in which the square resides
    private int col;

	/**
	 * @param value integer to which the square is set; 0 means undetermined
	 * @param row row in which the square resides, numbered 0 through NUM_ROWS
	 * @param col column in which the square resides, numbered 0 through NUM_COLUMNS
	 */
	public Square(int value, int row, int col)
	{
	    this.row = row;
	    this.col = col;
	    stateChanged = false;
        possibleValues = new ArrayList<Integer>();
        // if undetermined, set all integers 1 to 9 as
        // possible values for the square; otherwise, set
        // the possible values list to a single value
		if (value == 0){
    		for (int loop = 0; loop < NUM_DIGITS; loop++){
  				possibleValues.add( loop + 1 );
	 		}
		}
		else
			possibleValues.add( value );
	}

	/**
	 * @return row in which the square resides
	 */
	public int getRow()
	{
	    return row;
	}
	
	/**
	 * @return column in which the square resides
	 */
	public int getCol()
	{
	    return col;
	}
	
	/**
	 * @return Integer value that represents the value
	 * stored in the square; 0 means undetermined
	 */
	public Integer getValue()
	{
	    if ( possibleValues.size() == 1 )
	      return possibleValues.get(0);
	    else
	      return new Integer(0);
	}
	
	/**
	 * set the value of the square and update the state
	 * @param val int value to which the square is set
	 */
	public void setValue(int val)
	{
          stateChanged = true;
	      possibleValues = new ArrayList<Integer>();
	      possibleValues.add(val);
	}
	
	/**
	 * @return true if square has been modified since last update;
	 * false otherwise
	 */
    public boolean hasChanged(){
    	return stateChanged;
    }
    
	/**
	 * update the state of the square to false indicating that
	 * no change has been made to the square since the last update
	 */
    public void resetState(){
    	stateChanged = false;
    }

	/**
	 * removes an int value from the list of possible square values
	 * @param val int value to be removed
	 */
	public void removePossibleValue(int val)
	{
	    if ( possibleValues.size() == 1 )
	      return; // value has been determined
		possibleValues.remove( new Integer(val) );
		if ( possibleValues.size() == 1 )
		  stateChanged = true;
	}

	/**
	 * @return true if val is a possible value for this square; false otherwise
	 * @param val int value to be checked against square possible values
	 */
	public boolean hasPossibleValue(int val) 
	{
	    if ( possibleValues.contains( val) )
			return true;
		else
            return false;
	}

	/**
	 * @return copy of possible values for this square
	 */
	public ArrayList<Integer> getPossibleValues() 
	{
	    ArrayList<Integer> temp = new ArrayList<Integer>();
	    for (int pos = 0; pos < possibleValues.size(); pos++)
	    {
 	      temp.add( possibleValues.get(pos) );
	    }
	    return temp;
	}

	/**
	 * Override Object equals
	 * @return true if square has same value(s); false otherwise
	 */
	public boolean equals(Object obj)
	{
	   Square temp = (Square) obj;
	   ArrayList<Integer> list =  this.getPossibleValues();
	   ArrayList<Integer> list2 = temp.getPossibleValues();
   	   if ( list.size() == list2.size() )
   	   {
   	     for (int pos = 0; pos < list.size(); pos++)
   	     {
   	       int first = list.get(pos);
   	       int second = list2.get(pos);
   	       if ( first != second )
   	         return false;
   	     }
   	     return true;
   	   }
   	   return false;
	}
	
	/**
	 * @return String version of the square
	 */
	public String toString(){
		String temp = "(" + getRow() + ", " + getCol() + ")=" + getValue();
		return temp;
	}
}
