
/**
 * SudokuDisplay produces text output for the Sudoku board
 */
public class SudokuDisplay implements Sudoku
{
    // holds the possible values for the entire board,where 0 means
    // that corresponding value is not possible for the corresponding square
	private int[][] boardState;
	// Sudoku board squares to be displayed
    private Square[][] square;

  /**
   * Generates a text display of the current state of the Sudoku board
   * @param board the Sudoku board to be displayed
   */
  public SudokuDisplay(Board board)
  {
    // Future Project: abstract squares array to provide
    // information without providing direct access to squares
    this.square = board.getSquares();
    int possibleValuesWidth = NUM_SECTIONS_PER_TYPE*POSSIBILITIES_PER_ROW;
    int possibleValuesHeight = NUM_SECTIONS_PER_TYPE*POSSIBILITIES_PER_COL;

	boardState = new int[possibleValuesWidth][possibleValuesHeight];
  }

   /**
    * Each boardState position r,c corresponds to a square and a
    * particulat possible value for that square.
    * @return the particulat possible value for the square that
    * corresponds to the boardState location r,c if it exists; 0 otherwise
    * @param r row of a boardSate position
    * @param c column of a boardSate position
    */
   private int getPossibleValue( int r, int c )
   {
      return POSSIBILITIES_PER_ROW * r + c % POSSIBILITIES_PER_COL + 1;
   }

   /**
    * create a text display of the Sudoku board whose values are in squares
    */
   public void showBoard(){
	   for (int r = 0; r < boardState.length; r++){
		   for (int c = 0; c < boardState[0].length; c++){
		      // row of square whose boardState position is r,c
		      int squareRow = r / POSSIBILITIES_PER_ROW;
		      // column of square whose boardState position is r,c
		      int squareCol = c / POSSIBILITIES_PER_COL;
		      // row number for possible value between 0 and POSSIBILITIES_PER_ROW
		      int possRow = r % POSSIBILITIES_PER_ROW;
		      // column number for possible value between 0 and POSSIBILITIES_PER_COL
		      int possCol = c % POSSIBILITIES_PER_COL;
		    
		      // Each boardState position has a square (tempSquare) and a possible
		      // value for the square (possVal) associated with it
              Square tempSquare = square[squareRow][squareCol];
		      int possVal = getPossibleValue(possRow, possCol);
		      // Store corresponding boardState values based on tempSquare state
			  if ( tempSquare.getValue().equals( new Integer(0) ) &&
			       tempSquare.hasPossibleValue( possVal ) )
			     boardState[r][c] = possVal;
			  else if ( !tempSquare.getValue().equals( new Integer(0) ) &&
			            possRow == POSSIBILITIES_PER_ROW / 2 &&
			            possCol == POSSIBILITIES_PER_COL / 2 )
			     boardState[r][c] = tempSquare.getValue();
			  else
			     boardState[r][c] = EMPTY_VALUE;
		   }
	   }

	   // Use boardState values to display the Sudoku board
	   for (int r = 0; r < boardState.length; r++){
		   for (int c = 0; c < boardState[0].length; c++){
  		       if ( boardState[r][c] != 0 )
			      System.out.print( boardState[r][c] );
		       else
		          System.out.print(" ");
               if ( (c + 1) % POSSIBILITIES_PER_COL == 0)
               {
                 if ( (c + 1) % NUM_SECTIONS_PER_TYPE == 0 && c < NUM_SECTIONS_PER_TYPE * POSSIBILITIES_PER_COL - 1)
                   System.out.print("|");
                 else  if ( (c + 1) % NUM_SECTIONS_PER_TYPE != 0 )
                   System.out.print("!");
               }
		   }
		   if ( (r + 1) % POSSIBILITIES_PER_COL != 0) System.out.println();
		   else
		   {
		     if ( (r + 1) % NUM_SECTIONS_PER_TYPE == 0 && r < NUM_SECTIONS_PER_TYPE * POSSIBILITIES_PER_COL - 1)
		       System.out.println("\n====================================");
  		     else if (r < NUM_SECTIONS_PER_TYPE * POSSIBILITIES_PER_COL - 1)
  		       System.out.println("\n------------------------------------");
  		     else
  		       System.out.println();
  		   }
	   }
	   System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");

   }
}
